@@ -1,6 +1,5 @@
 import logging
 from collections import defaultdict
-from enum import Enum  # , EnumMeta, _EnumDict
 from functools import partial, wraps
 from gettext import gettext
 from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
@@ -14,7 +13,7 @@ from typer.core import TyperCommand, TyperGroup
 
 import gto
 from gto.exceptions import GTOException, WrongArgs
-from gto.ui import EMOJI_FAIL, EMOJI_GTO, bold, cli_echo, color, echo
+from gto.ui import EMOJI_FAIL, EMOJI_GTO, EMOJI_OK, bold, cli_echo, color, echo
 from gto.utils import format_echo, make_ready_to_serialize
 
 
@@ -185,6 +184,7 @@ app = Typer(
 arg_name = Argument(..., help="Artifact name")
 arg_version = Argument(..., help="Artifact version")
 arg_stage = Argument(..., help="Stage to promote to")
+
 option_rev = Option(None, "--rev", help="Repo revision to use", show_default=True)
 option_repo = Option(".", "-r", "--repo", help="Repository to use", show_default=True)
 option_all_branches = Option(
@@ -197,25 +197,13 @@ option_all_branches = Option(
 option_all_commits = Option(
     False, "-A", "--all-commits", is_flag=True, help="Read all commits"
 )
-
-
-class Format(str, Enum):
-    json = "json"
-    yaml = "yaml"
-
-
-class FormatDF(str, Enum):
-    json = "json"
-    yaml = "yaml"
-    table = "table"
-
-
+option_all = Option(False, "--all", "-a", help="Return all versions sorted")
 option_name = Option(None, "--name", "-n", help="Artifact name", show_default=True)
-option_sort = Option(
-    "desc",
-    "--sort",
-    "-s",
-    help="Desc for recent first, Asc for older first",
+option_ascending = Option(
+    False,
+    "--ascending",
+    "--asc",
+    help="Show new first",
     show_default=True,
 )
 option_expected = Option(
@@ -266,6 +254,13 @@ option_table = Option(
     help="Print output in table format",
     show_default=True,
 )
+option_registered_only = Option(
+    False,
+    "--registered-only",
+    "--ro",
+    is_flag=True,
+    help="Show only registered versions",
+)
 
 
 @app.callback("gto", invoke_without_command=True, no_args_is_help=True)
@@ -409,6 +404,9 @@ def register(
     version: Optional[str] = Option(
         None, "--version", "--ver", help="Version name in SemVer format"
     ),
+    message: Optional[str] = Option(
+        None, "--message", "-m", help="Message to annotate git tag with"
+    ),
     bump_major: bool = Option(
         False, "--bump-major", is_flag=True, help="Bump major version"
     ),
@@ -439,6 +437,7 @@ def register(
         name=name,
         ref=ref or "HEAD",
         version=version,
+        message=message,
         bump_major=bump_major,
         bump_minor=bump_minor,
         bump_patch=bump_patch,
@@ -457,6 +456,9 @@ def promote(
         "--version",
         help="If you provide REF, this will be used to name new version",
     ),
+    message: Optional[str] = Option(
+        None, "--message", "-m", help="Message to annotate git tag with"
+    ),
     simple: bool = Option(
         False,
         "--simple",
@@ -506,6 +508,7 @@ def promote(
         promote_version,
         ref,
         name_version,
+        message=message,
         simple=simple,
         force=force,
         skip_registration=skip_registration,
@@ -538,6 +541,9 @@ def which(
     name: str = arg_name,
     stage: str = arg_stage,
     ref: bool = option_ref_bool,
+    all: bool = option_all,
+    registered_only: bool = option_registered_only,
+    ascending: bool = option_ascending,
 ):
     """Find the latest artifact version in a given stage
 
@@ -547,9 +553,15 @@ def which(
         Print git tag that did the promotion:
         $ gto which nn prod --ref
     """
-    version = gto.api.find_promotion(repo, name, stage)
+    version = gto.api.find_versions_in_stage(
+        repo, name, stage, all=all, registered_only=registered_only
+    )
     if version:
-        if ref:
+        if all:
+            if ascending:
+                version.reverse()
+            format_echo([v.version for v in version], "lines")
+        elif ref:
             echo(version.tag or version.commit_hexsha)
         else:
             echo(version.version)
@@ -576,6 +588,7 @@ def parse_tag(
 def check_ref(
     repo: str = option_repo,
     ref: str = Argument(..., help="Git reference to analyze"),
+    json: bool = Option(False, "--json", is_flag=True, help="Output in JSON format"),
 ):
     """Find out the artifact version registered/promoted with ref
 
@@ -584,7 +597,19 @@ def check_ref(
         $ gto check-ref rf#prod
     """
     result = gto.api.check_ref(repo, ref)
-    format_echo(result, "json")
+    if json:
+        format_echo(result, "json")
+    else:
+        if result["version"]:
+            v = list(result["version"].values())[0]
+            echo(
+                f"""{EMOJI_OK} Version "{v["name"]}" of artifact "{v["artifact"]}" was registered"""
+            )
+        if result["stage"]:
+            s = list(result["stage"].values())[0]
+            echo(
+                f"""{EMOJI_OK} Version "{s["version"]}" of artifact "{s["artifact"]}" was promoted to "{s["stage"]}" stage"""
+            )
 
 
 @gto_command(section=CommandGroups.querying)
@@ -596,6 +621,7 @@ def show(
     json: bool = option_json,
     plain: bool = option_plain,
     name_only: bool = option_name_only,
+    registered_only: bool = option_registered_only,
 ):
     """Show the registry state
 
@@ -619,10 +645,11 @@ def show(
             name=name,
             all_branches=all_branches,
             all_commits=all_commits,
+            registered_only=registered_only,
             table=False,
         )
         if name_only:
-            format_echo(output, "lines")
+            format_echo([v["name"] for v in output], "lines")
         else:
             format_echo(output, "json")
     else:
@@ -632,6 +659,7 @@ def show(
                 name=name,
                 all_branches=all_branches,
                 all_commits=all_commits,
+                registered_only=registered_only,
                 table=True,
                 truncate_hexsha=True,
             ),
@@ -659,7 +687,7 @@ def history(
     all_commits: bool = option_all_commits,
     json: bool = option_json,
     plain: bool = option_plain,
-    sort: str = option_sort,
+    ascending: bool = option_ascending,
 ):
     """Show a journal of registry operations
 
@@ -677,7 +705,7 @@ def history(
                 name,
                 all_branches=all_branches,
                 all_commits=all_commits,
-                sort=sort,
+                ascending=ascending,
                 table=False,
             ),
             format="json",
@@ -689,7 +717,7 @@ def history(
                 name,
                 all_branches=all_branches,
                 all_commits=all_commits,
-                sort=sort,
+                ascending=ascending,
                 table=True,
                 truncate_hexsha=True,
             ),
