@@ -1,4 +1,3 @@
-import logging
 from datetime import datetime
 from enum import Enum
 from typing import FrozenSet, Iterable, Optional, Union
@@ -204,6 +203,7 @@ def version_from_tag(tag: git.Tag) -> BaseVersion:
         created_at=mtag.created_at,
         author=tag.tag.tagger.name,
         author_email=tag.tag.tagger.email,
+        message=tag.tag.message,
         commit_hexsha=tag.commit.hexsha,
         tag=tag.name,
     )
@@ -229,6 +229,7 @@ def promotion_from_tag(
                     created_at=mtag.created_at,
                     author=tag.tag.tagger.name,
                     author_email=tag.tag.tagger.email,
+                    message=tag.tag.message,
                     commit_hexsha=tag.commit.hexsha,
                 )
             )
@@ -240,6 +241,7 @@ def promotion_from_tag(
         created_at=mtag.created_at,
         author=tag.tag.tagger.name,
         author_email=tag.tag.tagger.email,
+        message=tag.tag.message,
         commit_hexsha=tag.commit.hexsha,
         tag=tag.name,
     )
@@ -289,9 +291,9 @@ class TagVersionManager(TagManager):
             art_name = parse_name(ref)[NAME]
             version_name = parse_name(ref)[VERSION]
         except (KeyError, ValueError, IndexError):
-            logging.warning(
-                "Provided ref doesn't exist or it is not a tag that registers a version"
-            )
+            # logging.warning(
+            #     "Provided ref doesn't exist or it is not a tag that registers a version"
+            # )
             return {}
         return {
             name: version
@@ -318,9 +320,9 @@ class TagStageManager(TagManager):
             _ = parse_name(ref)[STAGE]
             art_name = parse_name(ref)[NAME]
         except (KeyError, ValueError, IndexError):
-            logging.warning(
-                "Provided ref doesn't exist or it is not a tag that promotes to an stage"
-            )
+            # logging.warning(
+            #     "Provided ref doesn't exist or it is not a tag that promotes to an stage"
+            # )
             return {}
         return {
             name: promotion
