@@ -103,6 +103,11 @@ def test_commands(showcase):
         ["-r", path, "rf", "production"],
         "v1.2.4\n",
     )
+    _check_successful_cmd(
+        "which",
+        ["-r", path, "rf", "production", "--all", "--ascending"],
+        "v1.2.4\nv1.2.3\n",
+    )
     _check_successful_cmd(
         "which",
         ["-r", path, "rf", "production", "--ref"],
