@@ -191,8 +191,15 @@ def test_registry_state_tag_tag(showcase):
             "commit_hexsha",
             "promotions",
             "enrichments",
+            "message",
+        ],
+        "promotions": [
+            "author",
+            "author_email",
+            "created_at",
+            "commit_hexsha",
+            "message",
         ],
-        "promotions": ["author", "author_email", "created_at", "commit_hexsha"],
     }
 
     _check_state(state, EXPECTED_REGISTRY_TAG_TAG_STATE, exclude)
